/*-----------------------------------------------------------------------
**  Var1SimPs
**
**  Purpose:   Calculating impulse response function of a
**             VAR(1) system
**
**             x(t) = A * x(t-1) +  epsilon(t)
**
**  Usage:     xM = Var1SimPs(A,epsilon,Tbig);
**
**  Input:     A            - VAR(1) matrix, see above
**             epsilon        nx1 or 1xn vector of shocks in inital period, or Tbigxn
**                            matrix with shocks in all periods
**             Tbig           scalar, last period to calculate for
**
**
**
**  Paul.Soderlind@unisg.ch, Dec 2001
-----------------------------------------------------------------------*/

proc Var1SimPs(A,epsilon,Tbig);

  local n,xM,x1_t_1,t,epsilon2,x1;


  n = rows(A);

  if rows(epsilon)*cols(epsilon) == n;
    epsilon2       = zeros(Tbig,n);          /*non-zero only in first period*/
    epsilon2[1,.]  = vecr(epsilon)';         /*to accomodate Tbig=1*/
    epsilon        = epsilon2;
  endif;


  x1_t_1 = zeros(n,1);           /*starting vector*/
  xM     = miss(zeros(Tbig,n),0);    /*to put results in*/
  t = 1;
  do until t > Tbig;             /*loop over time periods*/
    x1      = A*x1_t_1 + epsilon[t,.]';
    xM[t,.] = x1';
    x1_t_1  = x1;
    t = t + 1;
  endo;

  retp( xM );

endp;
/*-----------------------------------------------------------------------*/

