/*-----------------------------------------------------------------------------
%  test2.prg
%
%  Testing solution algorithms for standard and robust case on model in
%  MonPol31. Commitment case.
%
%
%
%
%
%
%
%
%
%  Paolo Giordani and Paul Sderlind, February 2002
------------------------------------------------------------------------------*/

ErrDetProbIt = 1;
theta = 30;
/*----------------------------------------------------------------------------*/

library pgraph;              /*activate graphics library*/

dlibrary PsDtgSen;           /*making dll code available: commitment case*/

#include monpol31.prc;     /*a simple model*/
#include psvarsim.prc;     /*calculate IRF*/
#include PsTgSen.prc;      /*generalized Schur decomposition*/
#include PsGees.prc;       /*Schur decomposition*/
#include MonEEA.prc;       /*solution algorithms*/
#include MonRob.prc;
/*----------------------------------------------------------------------------*/

Tbig = 10;                 /*Periods to calculate impulse response fn for*/

bet   = 0.99;              /*Calvo model with preset prices*/
q1    = 0.75;       /*pi(t+1) = bet*E(t)pi(t+2) + delta*E(t)[fi*y(t+1)+epi(t+1)]*/
delta = q1/(1-q1)*(1-bet*(1-q1));
fi    = 2/7;        /*3/7 in lecture notes for 1999*/

gamm = 2;           /*psi should be approximately equal to 1/l2*/
                    /*a higher gamm gives more endogenous persistance*/

taupi = 0.5;        /*autocorrelation inflation shock*/
tauy  = 0.5;        /*autocorrelation output shock*/

cutoff = 1.0;


omega = 0.5;        /*reaction fn i(t)=omega*y(t)+chi*pi(t), [0.5]*/
chi   = 1.5;        /*[1.5]*/


{A,B,Q,U,R,F,n1,n2,upi,uy} = MonPol31(bet,delta,fi,gamm,tauy,taupi,
                                      omega,chi,diagrv(eye(3),1|0.5|0));

C =  eye(n1)|
     zeros(n2,n1);

upi = upi[1:n1];
uy  = uy[1:n1];
/*----------------------------------------------------------------------------*/

"\lCommitment, standard policy";
"[x1(t+1),p2(t+1)]  = M*[x1(t),p2(t)] + e(t+1)";
"[x2(t),u(t),p1(t)] = N*[x1(t),p2(t)]";

{M,N} = ComItAlg( A,B,Q,R,U,bet,n1,n2,cutoff );    /*commitment, standard policy*/

x1p2     = Var1SimPs(M,C*upi,Tbig);
x2up1    = x1p2*N';
Simupi_C = x2up1[.,1:3];            /*IRF of pi(t),y(t),i(t) to price shock*/

x1p2     = Var1SimPs(M,C*uy,Tbig);
x2up1    = x1p2*N';
Simuy_C  = x2up1[.,1:3];            /*IRF of pi(t),y(t),i(t) to output shock*/
/*----------------------------------------------------------------------------*/

" ";"Commitment, robust policy";
{M,N,Ma,Na,Fv,J0,J0a} = ComAlgR( A,B,Q,R,U,bet,n1,n2,cutoff,C,theta );


x1p2      = Var1SimPs(M,C*upi,Tbig);              /*worst case*/
x2up1     = x1p2*N';
Simupi_Cw = x2up1[.,1:3];           /*pi(t),y(t),i(t)*/

x1p2      = Var1SimPs(M,C*uy,Tbig);
x2up1     = x1p2*N';
Simuy_Cw  = x2up1[.,1:3];           /*pi(t),y(t),i(t)*/


x1p2      = Var1SimPs(Ma,C*upi,Tbig);             /*approximating model*/
x2up1     = x1p2*Na';
Simupi_Ca = x2up1[.,1:3];           /*pi(t),y(t),i(t)*/

x1p2      = Var1SimPs(Ma,C*uy,Tbig);
x2up1     = x1p2*Na';
Simuy_Ca  = x2up1[.,1:3];           /*pi(t),y(t),i(t)*/
/*----------------------------------------------------------------------------*/

tid = seqa(1,1,Tbig);

graphset;
begwind;
window( 2,2,1 );
  title("a. Commitment, IRF, price shock: pi");
  _plegstr = "Standard\000Worst\000Approx";
  _plegctl = 1;
  ytics(-2,3,1,0);
  xlabel("period");
  xy( tid,Simupi_C[.,1]~Simupi_Cw[.,1]~Simupi_Ca[.,1] );
nextwind;
  title("b. Commitment, IRF, price shock: y");
  xy( tid,Simupi_C[.,2]~Simupi_Cw[.,2]~Simupi_Ca[.,2] );
nextwind;
  title("c. Commitment, IRF, price shock: i");
  xy( tid,Simupi_C[.,3]~Simupi_Cw[.,3]~Simupi_Ca[.,3] );
endwind;
/*----------------------------------------------------------------------------*/


if ErrDetProbIt == 1;               /*calculate error detection probabilities*/


  sigma = { -0.04, -0.03, -0.02, -0.01 };

  NSim = 500;                 /*detection probabilities*/
  "\lCalculating error detection probabilities: be patient";
  {rho,JcM} = ErrDetProb(A,B,Q,R,U,bet,error(0),n1,n2,cutoff,C,142,NSim,sigma,123,2,
                          error(0),1e+4,0);

  graphset;
  begwind;
  window( 2,2,1 );
    title("MonPol31.m, detection probabilites");
    xlabel("sigma");
    xy( sigma,rho );
  nextwind;
    title("MonPol31.m, Loss fn values");
    xlabel("sigma");
    xy( sigma,JcM[.,2] );
  endwind;

endif;    /*ErrDetProbIt*/
/*----------------------------------------------------------------------------*/

dlibrary -d;                         /*unloading dll*/
end;

