/*-----------------------------------------------------------------------------
%  Program: test3.prg
%
%  Euler equation/IS plus Calvo equation. Discretionary solution or simple
%  rule solution, with Hansen-Sargent robustness.
%
%
%
%  Paolo Giordani and Paul.Soderlind, February 2002
%-----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
*
* Parameters to be set by user
*
*-----------------------------------------------------------------------------*/

discretion = 1;      @1 for discretionary solution, else simple rule@

theta = 55;         @ benchmark case, with R=0.2, T=150, discretion=55, simple rule=90@

ErrDetProbIt = 0;    @1 to activate detection error probability@
NSim = 1000;          @number of simulations for detection error probabilities@
T = 150;             @sample size for detection error probability@

Tbig  = 11;          @Periods to calculate impulse response fn for@

bet  = 0.99;
gamm = 0.5;
alfa = 0.645;                     /* parameter values  */
rho1 = 0.5;
rho2 = 0.5;

sig1 = 1;  /* std of output shock */
sig2 = 1;  /* std of inflation shocks */

ly   = 0.5;  /* loss function parameter: output */
li   = 0.2;   /* loss function parameter: interest rate */

cutoff = 1.0;


omega = 0.5;        /*reaction fn i(t)=omega*y(t)+chi*pi(t), [0.5]*/
chi   = 1.5;        /*[1.5]*/
/*----------------------------------------------------------------------------*/

library pgraph;                   /*activate graphics library*/

#include monpol31.prc;     /*a simple model*/
#include psvarsim.prc;     /*calculate IRF*/
#include PsTgSen.prc;      /*generalized Schur decomposition*/
#include PsGees.prc;       /*Schur decomposition*/
#include MonEEA.prc;       /*solution algorithms*/
#include MonRob.prc;

dlibrary PsdGees;                /*making dll code available: discretionary and simple rule case*/
/*----------------------------------------------------------------------------*/

screen off;
/*---------------------------------------------	State-space form ---------------------*/

n1 = 2;
n2 = 2;

A0=eye(n1+n2);
A0[3,4]=gamm;
A0[4,4]=bet;

A1=eye(n1+n2);
A1[1,1]=rho1;
A1[2,2]=rho2;
A1[3,1]=-1;
A1[4,2]=-1;
A1[4,3]=-alfa;

B1=zeros(n1+n2,1);
B1[3]=gamm;

C1=zeros(n1+n2,n1);     
C1[1,1]=sig1;
C1[2,2]=sig2;
         
A = inv(A0)*A1;
B = inv(A0)*B1;
C = inv(A0)*C1;

Q=zeros(n1+n2,n1+n2);
Q[3,3]=ly;
Q[4,4]=1;

R = li;
U = zeros(n1+n2,1);

upi = {0,1};   /* inflation shock */  
uy  = { 1, 0 };  /* output shock */
/*----------------------------------------------------------------------------*/

if discretion == 1;

                         /*standard solution*/

  {M,N,Ma,Na,Fu,Fv,J0,J0a} = DiscAlgR( A,B,Q,R,U,bet,n1,n2,eye(n1),zeros(n2,n1),
                                       1e-6,0,1,0,1,0,1e+4,C,1e+5);

  C1=C[1:n1,.];

  x1       = Var1SimPs(M,C1*upi,Tbig);
  x2       = x1*N';
  interest = -x1*Fu';
  Simupi_D = x2~interest;          /*IRF of y(t),pi(t),i(t) to price shock  */

                                       /*robust solution*/
  {M,N,Ma,Na,Fu,Fv,J0,J0a} = DiscAlgR( A,B,Q,R,U,bet,n1,n2,eye(n1),zeros(n2,n1),
                                       1e-6,0,1,0,1,0,1e+4,C,theta);

  x1        = Var1SimPs(M,C1*upi,Tbig);         /*worst case*/
  x2        = x1*N';
  interest  = -x1*Fu';
  Simupi_Dw = x2~interest;                      /*IRF of y(t),pi(t),i(t) to price shock*/

  x1        = Var1SimPs(Ma,C1*upi,Tbig);        /*approximating model*/
  x2        = x1*N';
  interest  = -x1*Fu';
  Simupi_Da = x2~interest;                      /*IRF of y(t),pi(t),i(t) to price shock*/

else;   @simple rule@

  "\lSimple rule";
  "x1(t+1) = M*x1(t)+ e(t+1)";
  "x2(t)   = N*x1(t)";

  Fu  = -(0~0~omega~chi);                    /*simple rule, standard policy*/
  {M,N,J0} = SimpRulT( A,B,Q,R,U,bet,n1,n2,Fu,eye(n1),zeros(n1,1),cutoff );

  C1=C[1:n1,.];

  x1 = Var1SimPs( M,C1*upi,Tbig );
  x2 = x1*N';
  x  = x1~x2;
  interest = -x*Fu';
  Simupi_D = x2~interest;                      /*IRF of y(t),pi(t),i(t) to price shock, standard*/

  x10 = zeros(n1,1);
  Fv = { 0 0 0 0, 0 0 0 0 };

  {M,N,Ma,Na,J0,J0a} = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,x10,cutoff,C,theta,Fv,1);
  " ";"J0 and J0a";(J0~J0a);

  Fv0 = zeros(n1,n1+n2);                         /*initial guess*/

  {M,N,Ma,Na,J0,J0a,Fv} = SimpAlgROpt( Fv0 );

  {M,N,Ma,Na,J0,J0a} = SimpAlgR( A,B,Q,R,U,bet,n1,n2,Fu,x10,cutoff,C,theta,Fv,1 );

  x1 = Var1SimPs( M,C1*upi,Tbig );
  x2 = x1*N';
  x  = x1~x2;
  interest = -x*Fu';
  Simupi_Dw = x2~interest;          /*worst case IRF of y(t),pi(t),i(t) to price shock*/

  x1 = Var1SimPs( Ma,C1*upi,Tbig ); /*approx mod IRF of y(t),pi(t),i(t) to price shock*/
  x2 = x1*N';
  x  = x1~x2;
  interest  = -x*Fu';
  Simupi_Da = x2~interest;

endif;
/*----------------------------------------------------------------------------*/

/*Plotting graph*/

xx=seqa(0,1,11);

graphset;
begwind;
window(2,2,0);
title("output");
date("");
_plegstr = "Standard RE\000Approximating\000Worst case";
_plegctl = 1;
  _pcsel   = 8;       /*dark grey seems to work best*/
  _pcolor  = { 8 };
  _pmcolor = { 8, 8, 8, 8, 8, 8, 8, 8, 15 };
  _ptitlht=0.4;
  _plwidth={4  4 4};
@  ytics(0, 3.5, 0.1,0.1);@
_plegctl={2 9 4.5 2 };
_paxht=0.4;
xy(xx,Simupi_D[.,1]~Simupi_Da[.,1]~Simupi_Dw[.,1]);

xut = xx~Simupi_D[.,1]~Simupi_Da[.,1]~Simupi_Dw[.,1];

nextwind;
title("inflation");
date("");
_plegstr = "Standard RE\000Approximating\000Worst case";
_plegctl = 1;
  _pcsel   = 8;       /*dark grey seems to work best*/
  _pcolor  = { 8 };
  _pmcolor = { 8, 8, 8, 8, 8, 8, 8, 8, 15 };
  _ptitlht=0.4;
  _plwidth={4  4 4};
@  ytics(-1.6, 0, 0.1,0.1); @
_plegctl={2 9 4.5 2 };
xy(xx,Simupi_D[.,2]~Simupi_Da[.,2]~Simupi_Dw[.,2]);
xut = xut~xx~Simupi_D[.,2]~Simupi_Da[.,2]~Simupi_Dw[.,2];

nextwind;
title("interest rate");
date("");
_plegstr = "Standard RE\000Approximating\000Worst case";
_plegctl = 1;
  _pcsel   = 8;       /*dark grey seems to work best*/
  _pcolor  = { 8 };
  _pmcolor = { 8, 8, 8, 8, 8, 8, 8, 8, 15 };
  _ptitlht=0.4;
  _plwidth={4  4 4};
@  ytics(0, 3.5, 0.1,0.1); @
_plegctl={2 9 4.5 2 };

xy(xx,Simupi_D[.,3]~Simupi_Da[.,3]~Simupi_Dw[.,3]);
xut = xut~xx~Simupi_D[.,3]~Simupi_Da[.,3]~Simupi_Dw[.,3];
endwind;


output file = test3.out reset;
outwidth 256;
xut;
output off;
/*----------------------------------------------------------------------------*/

if ErrDetProbIt == 1;
/* discretion */
 @ sigma =-1/40 | -1/50 | -1/55 |-1/60 | -1/70| -1/80 | -1/90|-1/100|-1/150| -1/180 | -1/190 | -1/200 | -1/300 | -1/400 | -1/500 | -1/1000 ; @

/* simple rule */
 sigma = -1/80 | -1/90| -1/95 |-1/100|-1/150| -1/180 | -1/190 | -1/200 | -1/300 | -1/400 | -1/500 | -1/1000 ;

  NaN=miss(1,1);
  if discretion == 1;
    model = 3;
    Fu    = NaN;
    Fv0   = NaN;
  else;
    model = 4;
  endif;
  constant = 0;

  "\lCalculating error detection probabilities: be patient";
  {rho,JcM} = ErrDetProb(A,B,Q,R,U,bet,Fu,n1,n2,cutoff,C,T,NSim,sigma,123,model,Fv0,1e+4,constant);

  graphset;
  title("Detection error probabilities");
  xy(sigma,rho);
endif;
/*----------------------------------------------------------------------------*/

dlibrary -d;                         /*unloading dll*/
end;

